package il.ac.tau.cs.sw1.rpn;

import java.util.ArrayDeque;
import java.util.Deque;
import java.util.Scanner;

/**
 * This class demonstrate a simple Reverse Polish Notation (RPN) calculator.
 */
public class RPNCalculator {

	public double evaluate(String expr) {
		return evaluate(new Scanner(expr));
	}

	private double evaluate(Scanner s) {
		Deque<Double> stack = new ArrayDeque<Double>();
		
		while (s.hasNext()) {							// while there are input tokens left
			if (s.hasNextDouble()) {					// if the token is a value	
				stack.push(s.nextDouble());				// push it onto the stack
			} else if (s.hasNext("[+\\-*/]")) {			// if the token is an operator
				double operand2 = stack.pop();			// pop the top values from the stack
				double operand1 = stack.pop();

				double result = 0;
				switch (s.next().charAt(0)) {			// evaluate the operator, with the values
				case '+':
					result = operand1 + operand2;
					break;
				case '-':
					result = operand1 - operand2;
					break;
				case '*':
					result = operand1 * operand2;
					break;
				case '/':
					result = operand1 / operand2;
					break;
				}
				stack.push(result);						// push the result back onto the stack
			}
		}
		return stack.pop();								// the result of the calculation
	}

	public static void main(String[] args) {
		RPNCalculator rpn = new RPNCalculator();
		Scanner s = new Scanner(System.in);
		System.out.print("> ");
		while (s.hasNextLine()) {
			System.out.println(rpn.evaluate(s.nextLine()));
			System.out.print("> ");
		}
	}
}
